@@ -14,15 +14,37 @@
 module Prim where
 import Internals
 import AST
+import Pretty
 
 op0 :: Ident -> [Val] -> Val
 op0 ","  = VList . concatMap vCast
-op0 "!"  = VJunc JNone
-op0 "&"  = VJunc JAll
-op0 "^"  = VJunc JOne
-op0 "|"  = VJunc JAny
+op0 "!"  = VJunc JNone . mkSet
+op0 "&"  = opJuncAll
+op0 "^"  = VJunc JOne . mkSet
+op0 "|"  = VJunc JAny . mkSet
 op0 s    = \x -> VError ("unimplemented listOp: " ++ s) (Val $ VList x)
 
+opJuncAll = opJunc (VJunc JAll) [(JAny, VJunc JAny), (JAll, VJunc JAll), (JNone, VJunc JNone)]
+
+opJunc :: (VJunc -> Val) -> [(JuncType, VJunc -> Val)] -> [Val] -> Val
+opJunc f fs vals
+    | (not . null) vals
+    , all isRight vals'
+    , (js, vs)  <- unzip $ map fromRight vals'
+    , [j]       <- nub js
+    , Just f'   <- lookup j fs
+    = f' $ unionManySets vs
+    | otherwise
+    = f $ mkSet vals
+    where
+    vals' :: [Either Val (JuncType, VJunc)]
+    vals' = map vCast vals
+
+isRight (Right _)    = True
+isRight (Left _)     = False
+fromRight (Right v)  = v
+fromRight (Left _)   = undefined
+
 op1 :: Ident -> (forall a. Context a => a) -> Val
 op1 "!"     = fmapVal not
 op1 "+"     = op1Numeric id
@@ -38,9 +60,10 @@ op1 "\\"    = VRef
 op1 "..."   = op1Range
 op1 "not"   = op1 "!"
 op1 "any"   = VJunc JAny
-op1 "all"   = VJunc JAll
+op1 "all"   = opJuncAll
 op1 "one"   = VJunc JOne
 op1 "none"  = VJunc JNone
+op1 "perl"  = \x -> VStr $ pretty (x :: Val)
 op1 s    = \x -> VError ("unimplemented unaryOp: " ++ s) (Val x)
 
 mapStr :: (Word8 -> Word8) -> [Word8] -> String
@@ -61,7 +84,7 @@ op2 "+>>"= op2Int shiftR
 op2 "~&" = op2Str $ mapStr2 (.&.)
 op2 "~<<"= \x y -> VStr $ mapStr (`shiftL` vCast y) (vCast x)
 op2 "~>>"= \x y -> VStr $ mapStr (`shiftR` vCast y) (vCast x)
-op2 "**" = op2Rat ((^) :: VRat -> VInt -> VRat)
+op2 "**" = op2Rat ((^^) :: VRat -> VInt -> VRat)
 op2 "+"  = op2Numeric (+)
 op2 "-"  = op2Numeric (-)
 op2 "~"  = op2Str (++)
@@ -89,13 +112,15 @@ op2 "&&" = op2Logical not
 op2 "||" = op2Logical (id :: Bool -> Bool)
 op2 "^^" = op2Bool ((/=) :: Bool -> Bool -> Bool)
 op2 "//" = op2Logical isJust
+op2 "!!" = op2Bool (\x y -> not x && not y)
 -- XXX pipe forward XXX
 op2 "and"= op2 "&&"
 op2 "or" = op2 "||"
 op2 "xor"= op2 "^^"
 op2 "err"= op2 "//"
+op2 "nor"= op2 "!!"
 op2 ";"  = \x y -> y -- XXX wrong! LoL!
-op2 s    = \x y -> VError ("unimplemented binaryOp: " ++ s) (App s [Val x, Val y])
+op2 s    = \x y -> VError ("unimplemented binaryOp: " ++ s) (App s [] [Val x, Val y])
 
 vCastStr :: Val -> VStr
 vCastStr = vCast
@@ -163,16 +188,23 @@ op2Numeric f x y
 type Symbol  = (String, Val)
 type Symbols = [Symbol]
 
-primOp :: String -> String -> [String] -> String -> Symbol
+primOp :: String -> String -> Params -> String -> Symbol
 primOp sym assoc prms ret = (name, sub)
     where
-    name = '&':fixity ++ ':':sym
-    sub  = VSub $ Sub SubMulti assoc prms ret (Prim f)
+    name = '&':'*':fixity ++ ':':sym
+    sub  = VSub $ Sub { isMulti     = True
+                      , subType     = SubRoutine
+                      , subAssoc    = assoc
+                      , subParams   = prms
+                      , subReturns  = ret
+                      , subFun      = (Prim f)
+                      }
     f :: [Val] -> Val
     f    = case arity of
-        0 -> op0 sym
+        0 -> \(x:_) -> op0 sym (vCast x)
         1 -> \[x]   -> op1 sym (vCast x)
         2 -> \[x,y] -> op2 sym (vCast x) (vCast y)
+        _ -> error (show arity)
     (arity, fixity) = case assoc of
         "pre"       -> (1, "prefix")
         "post"      -> (1, "postfix")
@@ -183,13 +215,21 @@ primOp sym assoc prms ret = (name, sub)
         "chain"     -> (2, "infix")
         "list"      -> (0, "infix")
         other       -> (0, other)
-
-primDecl str = primOp sym assoc prms ret
+        
+primDecl str = primOp sym assoc (reverse $ foldr foldParam [] prms) ret
     where
     (ret:assoc:sym:prms') = words str
     takeWord = takeWhile isAlphaNum . dropWhile (not . isAlphaNum)
     prms = map takeWord prms'
 
+doFoldParam cxt [] []       = [buildParam cxt "" "$a" (Val VUndef)]
+doFoldParam cxt [] (p:ps)   = (buildParam cxt "" (strInc $ paramName p) (Val VUndef):p:ps)
+doFoldParam cxt (s:name) ps = (buildParam cxt [s] name (Val VUndef) : ps)
+
+foldParam :: String -> Params -> Params
+foldParam "List" = doFoldParam "List" "*@x"
+foldParam x      = doFoldParam x ""
+
 -- XXX -- Junctive Types -- XXX --
 
 initSyms = map primDecl . filter (not . null) . lines $ "\
@@ -206,6 +246,7 @@ initSyms = map primDecl . filter (not . null) . lines $ "\
 \\n   Ref       pre     \\      (Any)\
 \\n   List      pre     ...     (Str|Num)\
 \\n   Bool      pre     not     (Bool)\
+\\n   Str       pre     perl    (List)\
 \\n   Junction  pre     any     (List)\
 \\n   Junction  pre     all     (List)\
 \\n   Junction  pre     one     (List)\
@@ -236,7 +277,7 @@ initSyms = map primDecl . filter (not . null) . lines $ "\
 \\n   Pair      non     =>      (Any, Any)\
 \\n   Int       non     cmp     (Str, Str)\
 \\n   Int       non     <=>     (Num, Num)\
-\\n   List      non     ..      ((Num, Num)|(Str,Str))\
+\\n   List      non     ..      (Any, Any)\
 \\n   Bool      chain   !=      (Num, Num)\
 \\n   Bool      chain   ==      (Num, Num)\
 \\n   Bool      chain   <       (Num, Num)\
@@ -250,6 +291,7 @@ initSyms = map primDecl . filter (not . null) . lines $ "\
 \\n   Bool      chain   gt      (Str, Str)\
 \\n   Bool      chain   ge      (Str, Str)\
 \\n   Scalar    left    &&      (Bool, Bool)\
+\\n   Scalar    left    !!      (Bool, Bool)\
 \\n   Scalar    left    ||      (Bool, Bool)\
 \\n   Scalar    left    ^^      (Bool, Bool)\
 \\n   Scalar    left    //      (Bool, Bool)\
